<?php
namespace Teknisa\PLA\Modules\Transportadas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla04100ServicosTransportados {
	protected $connection;
	protected $environment;
	protected $util;

	public function __construct($connection, $environment, $util) {
		$this->connection    = $connection;
		$this->environment   = $environment;
		$this->util          = $util;
	}
    
    public function returnComensais(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();

    	$sql = "SELECT SUM(QTCOMESTICAR) QTCOMESTICAR 
    	          FROM CARDADIA 
    			 WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI 
    			   AND DTCARDAPIO = :P_DTCARDAPIO";
	
		$params = array(
			"P_CDFILIAL"   => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_DTCARDAPIO" => $row["DTSERVTRAN"]
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
		return $result;
    }
    
    public function calculate(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();
    	$nrorg = $this->environment->getNrOrg();
    	$cdoperador = $this->environment->getCdOperador();
    	$concat = '';
		$filiServ = '';
		$filiServConcat = [];
    	if ($row['TODOS_SERV'] == 'N') {
    		foreach($row['NMSERVICO_CALC'] as $serv)  {
	            $concat .= $row['UNIDADE'].$serv.'_'; 
	        }
	        $filiServ = explode("_", $concat);
	        array_pop($filiServ);
    	} else {
			$sql = "SELECT SUBSTR(S.CDFILIAL || S.CDSERVFILI, 1, 9) AS FILISERV 
			          FROM SERVICO S, FILIOPER F 
			         WHERE S.CDFILIAL = :CDFILIAL 
			           AND((INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) 
			           AND (F.CDOPERADOR = :CDOPERADOR) 
			           AND (F.CDFILIAL = S.CDFILIAL) AND (S.NRORG = :NRORG) 
			         ORDER BY NLSSORT(S.CDFILIAL,'NLS_SORT=BINARY_AI') , NLSSORT(S.CDSERVFILI,'NLS_SORT=BINARY_AI')";
			
			$params = array(
				"CDOPERADOR"   => $cdoperador,
				"NRORG" => $nrorg,
				"CDFILIAL" => $row['UNIDADE'],
				"CDGRUPSERV" => 'T'
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
	    	$result = $stmt->fetchAll();
			foreach($result as $unServ) {
		        array_push($filiServConcat, $unServ['FILISERV']);
			}
		    $filiServ = $filiServConcat;
	    }

	    $nrlistaserv = $this->util->insertLista($filiServ);
	    $this->procedureServTransp($nrlistaserv, $row, $cdoperador);
    }
    
    public function calculateNotFoundRegister(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();
    	$nrorg = $this->environment->getNrOrg();
    	$cdoperador = $this->environment->getCdOperador();
    	
    	$row['UNIDADE'] = $row['CDFILIAL'];
    	$row['DTSERVTRAN_CALC'] = $row['DTSERVTRAN'];
    	
    	$concat = $row['CDFILIAL'].$row['CDSERVFILI']; 
    	$filiServ = explode("_", $concat);
    	
	    $nrlistaserv = $this->util->insertLista($filiServ);
	    $this->procedureServTransp($nrlistaserv, $row, $cdoperador);
    }
    
    public function procedureServTransp($nrlistaserv, $row, $cdoperador) {
    	$today = date("d/m/Y G:i:s");

    	$procedure = new StoredProcedure($this->connection, 'SERVICOS_TRANSPORTADOS');
    	
    	$procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $row['UNIDADE'],         Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_SERVICOS',   Param::PARAM_INPUT, $nrlistaserv,            Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $row['DTSERVTRAN_CALC'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $row['DTSERVTRAN_CALC'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador,             Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTHRCALCST', Param::PARAM_INPUT, $today,                  Param::PARAM_TYPE_STR));
    	$procedure->execute();
    	$this->util->deleteLista($nrlistaserv);
    }
    
    private function returnPlates($rows) {
    	$sqlPratos = "SELECT ST.NRSERVTRAN, ST.CDFILIAL, ST.CDSERVFILI, ST.DTSERVTRAN, ST.CDPRATO, ST.CDFILAUXPRAT, 
    						 ST.CDDIETPADST, PR.NMPRATO, NVL(PR.VRARREDTRANS, '0') VRARREDTRANS, ST.QTTOTAPRATST, 
    						 PR.SGUNIDADE, ST.CDDESTFILI, ST.CDCLIENTE, ST.QTCOMESTIST, ST.QTEFETCLIST, ST.QTCOMECLIST, 
    						 ST.QTPADRAOST, ST.QTCONTAIST, ST.QTTOTACONTST, ST.QTPARCPRATST, ST.QTCONTCOMPST, ST.VRPECONSST, 
    						 ST.NRSERVTRAN, ST.QTAMOSTRAST, ST.QTTOTACONTSTCAMOS, ST.QTCONTCOMPSTCAMOS, ST.NRORG 
    				    FROM SERVTRAN ST, PRATO PR 
    				   WHERE (ST.CDFILIAL    = :P_CDFILIAL)   
    				     AND (ST.CDSERVFILI  = :P_CDSERVFILI) 
    				     AND (ST.DTSERVTRAN  = :P_DTSERVTRAN) 
    				     AND (ST.CDDESTFILI = :P_CDDESTFILI) 
    				     AND (ST.CDCLIENTE = :P_CDCLIENTE) 
    				     AND (ST.CDPRATO      = PR.CDPRATO)   
    				     AND (ST.CDFILAUXPRAT = PR.CDFILAUXPRAT)
    				   ORDER BY ST.CDPRATO, ST.CDFILAUXPRAT, ST.CDCLIENTE, ST.CDDESTFILI";
		$params = array(
			"P_CDFILIAL"   => $rows['CDFILIAL'],
			"P_CDSERVFILI" => $rows['CDSERVFILI'],
			"P_DTSERVTRAN" => $rows['DTSERVTRAN'],
			"P_CDDESTFILI" => $rows['CDDESTFILI'],
			"P_CDCLIENTE"  => $rows['CDCLIENTE']
		);

		$stmt = $this->connection->prepare($sqlPratos);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
    }
	
	public function updateFat(DTO\Request $request, DTO\Response $response) {
		$rowsUpdate = $request->getDataSet()->getRows();

		$sqlUpdate = "UPDATE SERVTRAN 
		                 SET QTTOTAPRATST = :QTTOTAPRATST, QTCOMESTIST = :QTCOMESTIST, 
		                	 QTEFETCLIST = :QTEFETCLIST, QTTOTACONTST = :QTTOTACONTST, 
		                	 QTPARCPRATST = :QTPARCPRATST, QTCONTCOMPST = :QTCONTCOMPST, 
		                	 VRPECONSST = :VRPECONSST, QTAMOSTRAST = :QTAMOSTRAST, 
		                	 QTTOTACONTSTCAMOS = :QTTOTACONTSTCAMOS, 
		                	 QTCONTCOMPSTCAMOS = :QTCONTCOMPSTCAMOS 
		               WHERE CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI 
		                 AND DTSERVTRAN = :OLD_DTSERVTRAN AND CDPRATO = :OLD_CDPRATO 
		                 AND CDFILAUXPRAT = :OLD_CDFILAUXPRAT 
		                 AND CDDESTFILI = :OLD_CDDESTFILI AND CDCLIENTE = :OLD_CDCLIENTE 
		                 AND NRSERVTRAN = :OLD_NRSERVTRAN";
		                 
		foreach($rowsUpdate as $rows) {
			$plates = $this->returnPlates($rows);
			$menu = $this->verifyMenu($rows);
			$rows['QTEFETCLIST'] = $this->valueToFloat($rows['QTEFETCLIST']);
			
			foreach ($plates as $row) {
				foreach ($menu as $plate){
					if ($plate['CDPRATO'] == $row['CDPRATO']) {
						$vrPeconsCard = $plate['VRPECONSCARD'];
						$comensais = $plate['QTCOMESTICAR'];
						break;
					}
				};
				
				$recipePlate = $this->verifyRecipePlate($row);
				$percentage = $this->verifyPercentage($recipePlate, $row);

				$qtcomestist = ($rows['QTEFETCLIST'] * ($rows['COMENSAIS']/100));
				$qttotapratst = ($vrPeconsCard / 100 * $rows['QTEFETCLIST']) * $percentage[0]['QTPADRAO'];
				
				if ($percentage[0]['PERCENTUAL'] != NULL) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
	                     
	            if ($percentage[0]['QTPADRAO'] == NULL || $percentage[0]['QTPADRAO'] == 0 && $percentage[0]['QTCONTAI'] == NULL || $percentage[0]['QTCONTAI'] == 0) {
					if ($row['VRARREDTRANS'] != 0) {
						if ($qttotapratst > 0 && fmod($qttotapratst, $row['VRARREDTRANS']) > 0) $qttotapratst = (intval($qttotapratst/$row['VRARREDTRANS']) + 1) * $row['VRARREDTRANS'];
					}
	            }
				
				$qtcontcompstcamos = 0;
				
				if ($percentage[0]['QTCONTAI'] != NULL || $percentage[0]['QTCONTAI'] != 0) {
					if ($qttotapratst >= $percentage[0]['QTCONTAI']) {
					$qtparcpratst = fmod($qttotapratst, $percentage[0]['QTCONTAI']);
					$qttotacontst = intval($qttotapratst/$percentage[0]['QTCONTAI']); //18
					$qtcontcompst = $qttotacontst;
					
					if ($qtparcpratst > 0) {
						$qtcontincost = 1;
						$qttotacontst = $qttotacontst + 1;
						if ($percentage[0]['PERCENTUAL'] > 0) {
							if ($qtparcpratst < (($percentage[0]['PERCENTUAL']/100) * $percentage[0]['QTCONTAI'])) {
								$qtparcpratst = ($percentage[0]['QTCONTAI'] + $qtparcpratst) / 2;
								$qtcontincost = 2;
								$qtcontcompst = $qtcontcompst - 1;
							}
						}
					}
					} else {
						$qtparcpratst = $qttotapratst;
					    $qttotacontst = 1;
					    $qtcontcompst = 0;
					}
				} else {
					$qttotacontstcamos = NULL;
					$qtcontcompstcamos = NULL;
					$qttotacontst = 0;
					$qtparcpratst = 0;
					$qtcontcompst = 0;
				}
				
				if ($qtcomestist == 0 || $qtcomestist == null) $vrpeconsst = 0;
				else $vrpeconsst = $qtcomestist * 100 / $comensais;
				
				$qttotacontstcamos = (($qtcomestist * $row['QTPADRAOST']) + $row['QTAMOSTRAST'])/$row['QTCONTAIST'];
	
				if ($qtcontcompst > $row['QTCONTAIST']) {
					$qtcontincost = 1;
					$qtcontcompstcamos = $qttotacontstcamos;
				} else {
				    if ($percentage[0]['PERCENTUAL'] == null) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
				    else {
					    if ($percentage[0]['PERCENTUAL'] > 0) {
					    	if ($qtparcpratst < (($percentage[0]['PERCENTUAL'] / 100) * $row['QTCONTAIST'])){
					    		$qtcontincost = 2;
					    		$qtcontcompstcamos = $row['QTCONTCOMPSTCAMOS'] - 1;
					    	} else {
					    		$qtcontincost = 1;
					    		$qtcontcompstcamos = 0;
					    	}
					    }
				    }
				}
				
				$params = array(
					"QTTOTAPRATST"      => $qttotapratst,
					"QTCOMESTIST"       => $qtcomestist,
					"QTEFETCLIST"       => $rows['QTEFETCLIST'],
					"QTTOTACONTST"      => $qttotacontst,
					"QTPARCPRATST"      => $qtparcpratst,
					"QTCONTCOMPST"      => $qtcontcompst,
					"VRPECONSST"        => $vrpeconsst,
					"QTAMOSTRAST"       => $row['QTAMOSTRAST'],
					"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
					"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
					"OLD_CDFILIAL"      => $row['CDFILIAL'],
					"OLD_CDSERVFILI"    => $row['CDSERVFILI'],
					"OLD_DTSERVTRAN"    => $row['DTSERVTRAN'],
					"OLD_CDPRATO"       => $row['CDPRATO'],
					"OLD_CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
					"OLD_CDDESTFILI"    => $row['CDDESTFILI'],
					"OLD_CDCLIENTE"     => $row['CDCLIENTE'],
					"OLD_NRSERVTRAN"    => $row['NRSERVTRAN']
				);
	
				$stmt = $this->connection->prepare($sqlUpdate);
				$stmt->execute($params);
			    $this->procedureEtiquetran($params, $row);
			}
		}
		
	}
	
	private function verifyMenu($rows){
		$sql = "SELECT
		             CO.CDFILIAL,
		             CO.NRCARDAPIO,
		             CO.CDFILAUXPRAT,
		             CO.CDPRATO,
		             SUM(CO.VRPECONSCARD) AS VRPECONSCARD,
		             CA.CDSERVFILI,
		             CA.DTCARDAPIO,
		             SUM(CA.QTCOMESTICAR) AS QTCOMESTICAR,
		             NVL(CA.CDDIETPAD,'XXXX') AS CDDIETPAD,
		             NVL(PT.VRARREDTRANS, 0) AS VRARREDTRANS,
		             PT.SGUNIDADE
		          FROM
		             COMPCARD CO,
		             CARDADIA CA,
		             PRATO PT
		          WHERE
		             (CA.CDFILIAL   = :P_CDFILIAL)   AND
		             (CA.CDSERVFILI = :P_CDSERVFILI) AND
		             (CA.DTCARDAPIO = :P_DTSERVTRAN) AND
		             (CO.CDFILIAL   = CA.CDFILIAL)   AND
		             (CO.NRCARDAPIO = CA.NRCARDAPIO) AND
		             (CO.CDPRATO    = PT.CDPRATO)    AND
		             (CO.CDFILAUXPRAT = PT.CDFILAUXPRAT)
				GROUP BY CO.CDFILIAL, CO.NRCARDAPIO, CO.CDFILAUXPRAT, CO.CDPRATO, CA.CDSERVFILI, CA.DTCARDAPIO, NVL(CA.CDDIETPAD,'XXXX'), NVL(PT.VRARREDTRANS, 0), PT.SGUNIDADE";
		
		$params = array(
			"P_CDFILIAL"   => $rows['CDFILIAL'],
			"P_CDSERVFILI" => $rows['CDSERVFILI'],
			"P_DTSERVTRAN" => $rows['DTSERVTRAN']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	public function procedureEtiquetran($params, $row) {
		$procedure = new StoredProcedure($this->connection, 'GERA_ETIQUETRAN');

    	$procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL'],             Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $row['CDSERVFILI'],           Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDDIETPADST',  Param::PARAM_INPUT, 'XXXX',                       Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDDESTFILI',   Param::PARAM_INPUT, $row['CDDESTFILI'],           Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_NRSERVTRAN',   Param::PARAM_INPUT, $row['NRSERVTRAN'],           Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDCLIENTE',    Param::PARAM_INPUT, $row['CDCLIENTE'],            Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTSERVTRAN',   Param::PARAM_INPUT, $row['DTSERVTRAN'],           Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDFILAUXPRAT', Param::PARAM_INPUT, $row['CDFILAUXPRAT'],         Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDPRATO',      Param::PARAM_INPUT, $row['CDPRATO'],              Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTCOMPST', Param::PARAM_INPUT, $params['QTCONTCOMPST'],      Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTAIST',   Param::PARAM_INPUT, $row['QTCONTAIST'],           Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTINCOST', Param::PARAM_INPUT, $params['QTCONTCOMPSTCAMOS'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTPARCPRATST', Param::PARAM_INPUT, $params['QTPARCPRATST'],      Param::PARAM_TYPE_STR));
    	$procedure->execute();
	}
	
	private function verifyRecipePlate($row) {
		$sql = "SELECT 'RS' AS RECEITA FROM RECESERV WHERE CDFILIAL     = :P_CDFILIAL     AND CDSERVFILI   = :P_CDSERVFILI   AND CDPRATO      = :P_CDPRATO      AND CDFILAUXPRAT = :P_CDFILAUXPRAT UNION SELECT 'RL' AS RECEITA FROM RECEFILI RF WHERE RF.CDFILIAL     = :P_CDFILIAL     AND RF.CDPRATO      = :P_CDPRATO      AND RF.CDFILAUXPRAT = :P_CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL     = RF.CDFILIAL   AND RS.CDSERVFILI   = :P_CDSERVFILI AND RS.CDPRATO      = RF.CDPRATO    AND RS.CDFILAUXPRAT = RF.CDFILAUXPRAT) UNION SELECT 'RP' AS RECEITA FROM RECEPADR RP WHERE RP.CDPRATO      = :P_CDPRATO AND RP.CDFILAUXPRAT = :P_CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL     = :P_CDFILIAL   AND RS.CDSERVFILI   = :P_CDSERVFILI AND RS.CDPRATO      = RP.CDPRATO       AND RS.CDFILAUXPRAT = RP.CDFILAUXPRAT) AND NOT EXISTS (SELECT RF.CDPRATO FROM RECEFILI RF WHERE RF.CDFILIAL     = :P_CDFILIAL AND RF.CDPRATO      = RP.CDPRATO  AND RF.CDFILAUXPRAT = RP.CDFILAUXPRAT)";
		
		$params = array(
			"P_CDFILIAL"     => $row['CDFILIAL'],
			"P_CDSERVFILI"   => $row['CDSERVFILI'],
			"P_CDPRATO"      => $row['CDPRATO'],
			"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	private function verifyPercentage($recipePlate, $row) {
		if ($recipePlate[0]['RECEITA'] == 'RP') $sql = "SELECT QTPADRAOPRAT AS QTPADRAO, QTCONTAIPRAT AS QTCONTAI, QTAMOSTRPADR AS QTAMOSTRAUN, VRPEMINPADR  AS PERCENTUAL FROM PRATPADR WHERE CDPRATO      = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT";
		else if ($recipePlate[0]['RECEITA'] == 'RL') $sql = "SELECT QTPADRAOFILI AS QTPADRAO, QTCONTAIFILI AS QTCONTAI, QTAMOSTRFILI AS QTAMOSTRAUN, VRPEMINFILI  AS PERCENTUAL FROM PRATFILI WHERE CDPRATO      = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT";
		else if ($recipePlate[0]['RECEITA'] == 'RS') $sql = "SELECT QTPADRAOSERV AS QTPADRAO, QTCONTAISERV AS QTCONTAI, QTAMOSTRSERV AS QTAMOSTRAUN, VRPEMINSERV  AS PERCENTUAL FROM PRATSERV WHERE CDPRATO      = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT";
		
		$params = array(
			"P_CDPRATO"      => $row['CDPRATO'],
			"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	private function verifyPercentageGeneral($unidade) {
		$sql = "SELECT NVL(VRPEMINCONT, '0') VRPEMINCONT FROM PARAMPLANEJA WHERE CDFILIAL = :P_CDFILIAL";
		
		$params = array(
			"P_CDFILIAL" => $unidade
		);
	    
	    $result = $this->connection->fetchColumn($sql, $params);
	}
	
	public function verifyPlateInsert(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();

		$sql = "SELECT (CDPRATO || CDFILAUXPRAT) AS CDCHAVEPRATO, CDPRATO, 
						CDFILAUXPRAT, CDFILIAL, CDSERVFILI, CDDIETPADST, DTSERVTRAN 
				  FROM SERVTRAN 
				 WHERE (CDPRATO      = :P_CDPRATO)      
				   AND (CDFILAUXPRAT = :P_CDFILAUXPRAT) AND (CDFILIAL     = :P_CDFILIAL)     
				   AND (CDSERVFILI   = :P_CDSERVFILI)   AND (CDCLIENTE    = :P_CDCLIENTE)    
				   AND (CDDESTFILI   = :P_CDDESTFILI)   AND (DTSERVTRAN   = :P_DTSERVTRAN)";
		
		$params = array(
			"P_CDPRATO"      => $row['CDPRATO'],
			"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
			"P_CDFILIAL"     => $row['CDFILIAL'],
			"P_CDSERVFILI"   => $row['CDSERVFILI'],
			"P_CDCLIENTE"    => $row['CDCLIENTE'],
			"P_CDDESTFILI"   => $row['CDDESTFILI'],
			"P_DTSERVTRAN"   => $row['DTSERVTRAN']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    if (empty($result) == true) $this->searchNewCodeInsert($row);
	    else $response->addDataSet(new DataSet("result", $result));
	}
	
	private function searchNewCodeInsert($row) {
		$sql = "SELECT MAX(NRSERVTRAN) + 1 AS SEQUSERV 
		          FROM SERVTRAN 
		         WHERE (CDFILIAL    = :P_CDFILIAL)    AND (CDSERVFILI  = :P_CDSERVFILI)  
		           AND (DTSERVTRAN  = :P_DTSERVTRAN)  AND (CDDIETPADST = :P_CDDIETPADST)";
		
		$params = array(
			"P_CDFILIAL"    => $row['CDFILIAL'],
			"P_CDSERVFILI"  => $row['CDSERVFILI'],
			"P_DTSERVTRAN"  => $row['DTSERVTRAN'],
			"P_CDDIETPADST" => 'XXXX'
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $codeSeq = $stmt->fetchAll();
	    $this->insert($row, $codeSeq);
	}
	
	private function insert($row, $codeSeq) {
		$sql = "INSERT INTO SERVTRAN 
					(CDFILIAL, CDSERVFILI, DTSERVTRAN, CDPRATO, CDFILAUXPRAT, CDDIETPADST, 
					 QTTOTAPRATST, CDDESTFILI, CDCLIENTE, QTCOMESTIST, QTCOMECLIST, QTPADRAOST,
					 QTCONTAIST, QTTOTACONTST, QTPARCPRATST, QTCONTCOMPST, VRPECONSST, NRSERVTRAN,
					 QTAMOSTRAST, QTTOTACONTSTCAMOS, QTCONTCOMPSTCAMOS, QTEFETCLIST) 
				VALUES (:CDFILIAL, :CDSERVFILI, :DTSERVTRAN, :CDPRATO, :CDFILAUXPRAT, :CDDIETPADST, 
						:QTTOTAPRATST, :CDDESTFILI, :CDCLIENTE, :QTCOMESTIST, :QTCOMECLIST, :QTPADRAOST, 
						:QTCONTAIST, :QTTOTACONTST, :QTPARCPRATST, :QTCONTCOMPST, :VRPECONSST, :NRSERVTRAN, 
						:QTAMOSTRAST, :QTTOTACONTSTCAMOS, :QTCONTCOMPSTCAMOS, :QTEFETCLIST)";
		
		$row['QTCOMESTIST'] = $this->valueToFloat($row['QTCOMESTIST']);
		$row['QTCOMECLIST'] = $this->valueToFloat($row['QTCOMECLIST']);
		
		$comensais = $row['QTCOMECLIST'];
		$vrPeconsCard = 100 * $row['QTCOMESTIST'] / $comensais;
		$row['VRARREDTRANS'] = $vrPeconsCard;

		$menu = $this->verifyMenu($row);
		$recipePlate = $this->verifyRecipePlate($row);
		$percentage = $this->verifyPercentage($recipePlate, $row);
		
		foreach ($menu as $plate){
			if ($plate['CDPRATO'] == $row['CDPRATO']) {
				$vrPeconsCard = $plate['VRPECONSCARD'];
				$comensais = $plate['QTCOMESTICAR'];
				break;
			}
		};
		
		$row['QTCOMESTIST'] = $this->valueToFloat($row['QTCOMESTIST']);
		$row['QTCOMECLIST'] = $this->valueToFloat($row['QTCOMECLIST']);
		$row['QTAMOSTRAST'] = $this->valueToFloat($row['QTAMOSTRAST']);

		$qtcomestist = $row['QTCOMESTIST'];
		$qttotapratst = intval($percentage[0]['QTPADRAO'] * $row['QTCOMESTIST']);

		if ($percentage[0]['PERCENTUAL'] != NULL) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
                     
        if ($percentage[0]['QTPADRAO'] == NULL || $percentage[0]['QTPADRAO'] == 0 && $percentage[0]['QTCONTAI'] == NULL || $percentage[0]['QTCONTAI'] == 0) {
			if ($row['VRARREDTRANS'] != 0) {
				if ($qttotapratst > 0 && fmod($qttotapratst, $row['VRARREDTRANS']) > 0) $qttotapratst = (intval($qttotapratst/$row['VRARREDTRANS']) + 1) * $row['VRARREDTRANS'];
			}
        }
		
		if ($percentage[0]['QTPADRAO'] == NULL) $qtpadraost = 0;
		else $qtpadraost = $percentage[0]['QTPADRAO'];
		
		$code = str_pad($codeSeq[0]['SEQUSERV'] + 1 , 10, '0' , STR_PAD_LEFT);
		
		$qtcontcompstcamos = 0;
		
		if ($percentage[0]['QTCONTAI'] != NULL || $percentage[0]['QTCONTAI'] != 0) {
			if ($qttotapratst >= $percentage[0]['QTCONTAI']) {
				$qtparcpratst = fmod($qttotapratst, $percentage[0]['QTCONTAI']);
				$qttotacontst = intval($qttotapratst/$percentage[0]['QTCONTAI']); //18
				$qtcontcompst = $qttotacontst;
				
				if ($qtparcpratst > 0) {
					$qtcontincost = 1;
					$qttotacontst = $qttotacontst + 1;
					if ($percentage[0]['PERCENTUAL'] > 0) {
						if ($qtparcpratst < (($percentage[0]['PERCENTUAL']/100) * $percentage[0]['QTCONTAI'])) {
							$qtparcpratst = ($percentage[0]['QTCONTAI'] + $qtparcpratst) / 2;
							$qtcontincost = 2;
							$qtcontcompst = $qtcontcompst - 1;
						}
					}
				}
			} else {
				$qtparcpratst = $qttotapratst;
			    $qttotacontst = 1;
			    $qtcontcompst = 0;
			}
		} else {
			$qttotacontstcamos = NULL;
			$qtcontcompstcamos = NULL;
			$qttotacontst = 0;
			$qtparcpratst = 0;
			$qtcontcompst = 0;
		}
			
		if ($qtcomestist == 0 || $qtcomestist == null) $vrpeconsst = 0;
		else $vrpeconsst = $qtcomestist * 100 / $comensais;
		
		if ($percentage[0]['QTPADRAO'] != null || $percentage[0]['QTPADRAO'] != 0) $qtpadraost =$percentage[0]['QTPADRAO'];
		else $qtpadraost = $row['QTPADRAOST'];
		
		if ($percentage[0]['QTCONTAI'] != null || $percentage[0]['QTCONTAI'] != 0) $qtcontaist =$percentage[0]['QTCONTAI'];
		else $qtcontaist = $row['QTCONTAIST'];
		
		$qttotacontstcamos = (($qtcomestist * $qtpadraost) + $row['QTAMOSTRAST'])/$qtcontaist;

		if ($qtcontcompst > $qtpadraost) {
			$qtcontincost = 1;
			$qtcontcompstcamos = $qttotacontstcamos;
		} else {
		    if ($percentage[0]['PERCENTUAL'] == null) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
		    else {
			    if ($percentage[0]['PERCENTUAL'] > 0) {
			    	if ($qtparcpratst < (($percentage[0]['PERCENTUAL'] / 100) * $qtpadraost)){
			    		$qtcontincost = 2;
			    		$qtcontcompstcamos = $row['QTCONTCOMPSTCAMOS'] - 1;
			    	} else {
			    		$qtcontincost = 1;
			    		$qtcontcompstcamos = 0;
			    	}
			    }
		    }
		}
		
		$params = array(
			"CDFILIAL"          => $row['CDFILIAL'],
			"CDSERVFILI"        => $row['CDSERVFILI'],
			"DTSERVTRAN"        => $row['DTSERVTRAN'],
			"CDPRATO"           => $row['CDPRATO'],
			"CDFILAUXPRAT"      => $row['CDFILAUXPRAT'],
			"CDDIETPADST"       => 'XXXX',
			"QTTOTAPRATST"      => $qttotapratst,
			"CDDESTFILI"        => $row['CDDESTFILI'],
			"CDCLIENTE"         => $row['CDCLIENTE'],
			"QTCOMESTIST"       => $qtcomestist,
			"QTCOMECLIST"       => $row['QTCOMECLIST'],
			"QTPADRAOST"        => $qtpadraost,
			"QTCONTAIST"        => $qtcontaist,
			"QTTOTACONTST"      => $qttotacontst,
			"QTPARCPRATST"      => $qtparcpratst,
			"QTCONTCOMPST"      => $qtcontcompst,
			"VRPECONSST"        => $vrpeconsst,
			"NRSERVTRAN"        => $code,
			"QTAMOSTRAST"       => $row['QTAMOSTRAST'],
			"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
			"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
			"QTEFETCLIST"       => $row['QTEFETCLIST']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->procedureEtiquetranInsert($params);
	}
	
	public function procedureEtiquetranInsert($params) {
		try {
			$this->connection->beginTransaction();
			$procedure = new StoredProcedure($this->connection, 'GERA_ETIQUETRAN');
	
	    	$procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $params['CDFILIAL'],             Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $params['CDSERVFILI'],           Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDDIETPADST',  Param::PARAM_INPUT, 'XXXX',                          Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDDESTFILI',   Param::PARAM_INPUT, $params['CDDESTFILI'],           Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_NRSERVTRAN',   Param::PARAM_INPUT, $params['NRSERVTRAN'],           Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDCLIENTE',    Param::PARAM_INPUT, $params['CDCLIENTE'],            Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_DTSERVTRAN',   Param::PARAM_INPUT, $params['DTSERVTRAN'],           Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDFILAUXPRAT', Param::PARAM_INPUT, $params['CDFILAUXPRAT'],         Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_CDPRATO',      Param::PARAM_INPUT, $params['CDPRATO'],              Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_QTCONTCOMPST', Param::PARAM_INPUT, $params['QTCONTCOMPST'],         Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_QTCONTAIST',   Param::PARAM_INPUT, $params['QTCONTAIST'],           Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_QTCONTINCOST', Param::PARAM_INPUT, $params['QTCONTCOMPSTCAMOS'],    Param::PARAM_TYPE_STR));
	    	$procedure->addParam(new Param('P_QTPARCPRATST', Param::PARAM_INPUT, $params['QTPARCPRATST'],         Param::PARAM_TYPE_STR));
	    	$procedure->execute();
		} catch (\Exception $e) {
	    	$this->connection->rollback();
		}
	}
	
	public function deleteEtiqueta(DTO\Request $request, DTO\Response $response) {
		$row = $request->getDataSet()->getRows();
		$sql = "DELETE FROM ETIQUETRAN 
		         WHERE ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI )
		           AND ( CDDESTFILI = :CDDESTFILI ) AND ( CDCLIENTE = :CDCLIENTE ) 
		           AND ( DTSERVTRAN = :DTSERVTRAN ) AND ( DTSERVTRAN = :DTSERVTRAN )";
		
		$params = array(
			"CDFILIAL"   => $row[0]['CDFILIAL'],
			"CDSERVFILI" => $row[0]['CDSERVFILI'],
			"CDDESTFILI" => $row[0]['CDDESTFILI'],
			"CDCLIENTE"  => $row[0]['CDCLIENTE'],
			"DTSERVTRAN" => $row[0]['DTSERVTRAN']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->deleteServtran($row);
	}
	
	private function deleteServtran($row) {
		$sql = "DELETE FROM SERVTRAN 
				 WHERE (CDFILIAL     = :P_CDFILIAL) AND (CDSERVFILI   = :P_CDSERVFILI) 
				   AND (DTSERVTRAN   = :P_DTSERVTRAN) AND (CDDESTFILI   = :P_CDDESTFILI) 
				   AND (CDCLIENTE    = :P_CDCLIENTE)";
		
		$params = array(
			"P_CDFILIAL"   => $row[0]['CDFILIAL'],
			"P_CDSERVFILI" => $row[0]['CDSERVFILI'],
			"P_DTSERVTRAN" => $row[0]['DTSERVTRAN'],
			"P_CDDESTFILI" => $row[0]['CDDESTFILI'],
			"P_CDCLIENTE"  => $row[0]['CDCLIENTE']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	}
	
	public function updatePlate(DTO\Request $request, DTO\Response $response) {
		$rows = $request->getDataSet()->getRows();
		$sqlUpdate = "UPDATE SERVTRAN 
						 SET QTTOTAPRATST = :QTTOTAPRATST, QTCOMESTIST = :QTCOMESTIST, 
							 QTEFETCLIST = :QTEFETCLIST, QTTOTACONTST = :QTTOTACONTST, 
							 QTPARCPRATST = :QTPARCPRATST, QTCONTCOMPST = :QTCONTCOMPST, 
							 VRPECONSST = :VRPECONSST, QTAMOSTRAST = :QTAMOSTRAST, 
							 QTTOTACONTSTCAMOS = :QTTOTACONTSTCAMOS, 
							 QTCONTCOMPSTCAMOS = :QTCONTCOMPSTCAMOS,
							 QTPADRAOST = :QTPADRAOST, QTCONTAIST = :QTCONTAIST
					   WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI 
						 AND DTSERVTRAN = :DTSERVTRAN AND CDPRATO = :CDPRATO 
						 AND CDFILAUXPRAT = :CDFILAUXPRAT 
						 AND CDDESTFILI = :CDDESTFILI AND CDCLIENTE = :CDCLIENTE 
						 AND NRSERVTRAN = :NRSERVTRAN";
		
		foreach ($rows as $row) {
			$menu = $this->verifyMenu($row);
			$recipePlate = $this->verifyRecipePlate($row);
			$percentage = $this->verifyPercentage($recipePlate, $row);
			$comensais = $row['QTCOMECLIST'];
			foreach ($menu as $plate){
				if ($plate['CDPRATO'] == $row['CDPRATO']) {
					$vrPeconsCard = $plate['VRPECONSCARD'];
					$comensais = $plate['QTCOMESTICAR'];
					break;
				}
			};
			
			$row['COMENSAIS'] = $this->valueToFloat($row['COMENSAIS']);
			$row['QTPADRAOST'] = $this->valueToFloat($row['QTPADRAOST']);
			$row['QTCONTAIST'] = $this->valueToFloat($row['QTCONTAIST']);
			
			$qttotapratst = $row['COMENSAIS'] * $row['QTPADRAOST'];
			$qtcontcompst = $qttotapratst/$row['QTCONTAIST'];
			if ($row['COMENSAIS'] == 0 || $row['COMENSAIS'] == null) $vrpeconsst = 0;
			else $vrpeconsst = $row['COMENSAIS'] * 100 / $comensais;
			
			$qttotacontstcamos = (($row['COMENSAIS'] * $row['QTPADRAOST']) + $this->valueToFloat($row['QTAMOSTRAST']))/$row['QTCONTAIST'];

			if ($qttotapratst >= $row['QTCONTAIST']) {
				$qtparcpratst = fmod($qttotapratst, $row['QTCONTAIST']);
				$qttotacontst = intval($qttotapratst/$row['QTCONTAIST']); //18
				$qtcontcompst = $qttotacontst;
				$qtcontcompstcamos = $qttotacontst;
				
				if ($qtparcpratst > 0) {
					$qtcontincost = 1;
					$qtcontincostcamos = 1;
					$qttotacontst = $qttotacontst + 1;
					if ($percentage[0]['PERCENTUAL'] > 0) {
						if ($qtparcpratst < (($percentage[0]['PERCENTUAL']/100) * $row['QTCONTAIST'])) {
							$qtparcpratst = ($row['QTCONTAIST'] + $qtparcpratst) / 2;
							$qtcontincost = 2;
							$qtcontcompst = $qtcontcompst - 1;
							$qtparcpratstcamos = $qtparcpratst;
							$qtcontincostcamos = 2;
							$qtcontcompstcamos = $qttotacontst - 1;
						}
					} else {
						$qtcontincostcamos = 2;
						$qtcontcompstcamos = 1;
					}
				}
			} else {
				$qtparcpratst = $qttotapratst;
			    $qttotacontst = 1;
			    $qtcontcompst = 0;
			    $qtcontincost = 1;
			    $qtcontcompstcamos = 1;
			    $qttotacontstcamos = 2;
			}
			// if ($qtcontcompst > $row['QTCONTAIST']) {
			// 	$qtcontincost = 1;
			// 	$qtcontcompstcamos = $qttotacontstcamos;
			// } else {
			//     if ($percentage[0]['PERCENTUAL'] == null) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
			//     else {
			// 	    if ($percentage[0]['PERCENTUAL'] > 0) {
			// 	    	if ($qtparcpratst < (($percentage[0]['PERCENTUAL'] / 100) * $percentage[0]['QTCONTAI'])){
			// 	    		$qtcontincost = 2;
			// 	    		$qtcontcompstcamos = $qtcontcompst - 1;
			// 	    	} else {
			// 	    		$qtcontincost = 1;
			// 	    		$qtcontcompstcamos = 0;
			// 	    	}
			// 	    }
			//     }
			// }
			
			$params = array(
				"QTTOTAPRATST"      => $qttotapratst,
				"QTCOMESTIST"       => $row['COMENSAIS'],
				"QTEFETCLIST"       => $row['QTEFETCLIST'],
				"QTTOTACONTST"      => $qttotacontst,
				"QTPARCPRATST"      => $qtparcpratst,
				"QTCONTCOMPST"      => $qtcontcompst,
				"VRPECONSST"        => $vrpeconsst,
				"QTAMOSTRAST"       => $this->valueToFloat($row['QTAMOSTRAST']),
				"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
				"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
				"CDFILIAL"          => $row['CDFILIAL'],
				"CDSERVFILI"        => $row['CDSERVFILI'],
				"DTSERVTRAN"        => $row['DTSERVTRAN'],
				"CDPRATO"           => $row['CDPRATO'],
				"CDFILAUXPRAT"      => $row['CDFILAUXPRAT'],
				"CDDESTFILI"        => $row['CDDESTFILI'],
				"CDCLIENTE"         => $row['CDCLIENTE'],
				"NRSERVTRAN"        => $row['NRSERVTRAN'],
				"QTPADRAOST"        => $row['QTPADRAOST'], 
				"QTCONTAIST"        => $row['QTCONTAIST']
			);
			
			$stmt = $this->connection->prepare($sqlUpdate);
			$stmt->execute($params);
		    $this->procedureEtiquetranInsert($params);
		}
	}
	
	public function deletePlate(DTO\Request $request, DTO\Response $response) {
		$row = $request->getDataSet()->getRows();
		$sql = "DELETE FROM SERVTRAN 
				 WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDSERVFILI = :P_CDSERVFILI ) 
				   AND ( DTSERVTRAN = :P_DTSERVTRAN ) AND ( CDDESTFILI = :P_CDDESTFILI ) 
				   AND ( CDCLIENTE = :P_CDCLIENTE ) AND ( NRSERVTRAN = :P_NRSERVTRAN )";

		$params = array(
			"P_CDFILIAL"   => $row[0]['CDFILIAL'],
			"P_CDSERVFILI" => $row[0]['CDSERVFILI'],
			"P_DTSERVTRAN" => $row[0]['DTSERVTRAN'],
			"P_CDDESTFILI" => $row[0]['CDDESTFILI'],
			"P_CDCLIENTE"  => $row[0]['CDCLIENTE'],
			"P_NRSERVTRAN" => $row[0]['NRSERVTRAN']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	}
	
	function valueToFloat($value){
        if(is_string($value) && !$this->isEmptyOrBlank($value)){
            $dotPos   = strpos($value, ".");
            $commaPos = strpos($value, ",");
            if($commaPos == false && $dotPos == false){
                return floatval($value);
            } 
            $dotPos   = $dotPos   !== false ? $dotPos   : -1;
            $commaPos = $commaPos !== false ? $commaPos : -1;
            if($commaPos > $dotPos){
                $firstReplace = str_replace(".", "", $value);
                $secondReplace = str_replace(",", ".", $firstReplace);
                return floatval($secondReplace);
            } 
            if($commaPos < $dotPos){
                $firstReplace = str_replace(",", "", $value);
                return floatval($firstReplace);
            }
        } else {
            return $value;
        }
    }

    function isEmptyOrBlank($value){
        return !isset($value) || $this->isBlank($value);
    }
    
    function isBlank($value){
        $newStr = str_replace(" ", "", $value);
        return strlen($newStr) === 0;
    }
}